import * as AWS from 'aws-sdk';
import { ISDK } from './aws-auth';
export interface ListStackResources {
    listStackResources(): Promise<AWS.CloudFormation.StackResourceSummary[]>;
}
export declare class LazyListStackResources implements ListStackResources {
    private readonly sdk;
    private readonly stackName;
    private stackResources;
    constructor(sdk: ISDK, stackName: string);
    listStackResources(): Promise<AWS.CloudFormation.StackResourceSummary[]>;
    private getStackResources;
}
export declare class CfnEvaluationException extends Error {
}
export interface ResourceDefinition {
    readonly LogicalId: string;
    readonly Type: string;
    readonly Properties: {
        [p: string]: any;
    };
}
export interface EvaluateCloudFormationTemplateProps {
    readonly template: Template;
    readonly parameters: {
        [parameterName: string]: string;
    };
    readonly account: string;
    readonly region: string;
    readonly partition: string;
    readonly urlSuffix: (region: string) => string;
    readonly listStackResources: ListStackResources;
}
export declare class EvaluateCloudFormationTemplate {
    private readonly stackResources;
    private readonly template;
    private readonly context;
    private readonly account;
    private readonly region;
    private readonly partition;
    private readonly urlSuffix;
    private cachedUrlSuffix;
    constructor(props: EvaluateCloudFormationTemplateProps);
    createNestedEvaluateCloudFormationTemplate(listNestedStackResources: ListStackResources, nestedTemplate: Template, nestedStackParameters: {
        [parameterName: string]: any;
    }): EvaluateCloudFormationTemplate;
    establishResourcePhysicalName(logicalId: string, physicalNameInCfnTemplate: any): Promise<string | undefined>;
    findPhysicalNameFor(logicalId: string): Promise<string | undefined>;
    findLogicalIdForPhysicalName(physicalName: string): Promise<string | undefined>;
    findReferencesTo(logicalId: string): Array<ResourceDefinition>;
    evaluateCfnExpression(cfnExpression: any): Promise<any>;
    private references;
    private parseIntrinsic;
    private findRefTarget;
    private findGetAttTarget;
    private formatResourceAttribute;
    private getServiceOfResource;
    private getResourceTypeArnPartOfResource;
}
export declare type Template = {
    [section: string]: {
        [headings: string]: any;
    };
};
