import * as cxapi from '@aws-cdk/cx-api';
import { ISDK } from './aws-auth';
import { Template } from './util/cloudformation';
/**
 * Reads the currently deployed template from CloudFormation and adds a
 * property, `NestedTemplate`, to any nested stacks that appear in either
 * the deployed template or the newly synthesized template. `NestedTemplate`
 * is populated with contents of the nested template by mutating the
 * `template` property of `rootStackArtifact`. This is done for all
 * nested stack resources to arbitrary depths.
 */
export declare function loadCurrentTemplateWithNestedStacks(rootStackArtifact: cxapi.CloudFormationStackArtifact, sdk: ISDK): Promise<TemplateWithNestedStackNames>;
/**
 * Returns the currently deployed template from CloudFormation that corresponds to `stackArtifact`.
 */
export declare function loadCurrentTemplate(stackArtifact: cxapi.CloudFormationStackArtifact, sdk: ISDK): Promise<Template>;
export interface TemplateWithNestedStackNames {
    readonly deployedTemplate: Template;
    readonly nestedStackNames: {
        [nestedStackLogicalId: string]: NestedStackNames;
    };
}
export interface NestedStackNames {
    readonly nestedStackPhysicalName: string | undefined;
    readonly nestedChildStackNames: {
        [logicalId: string]: NestedStackNames;
    };
}
